@@ -1,6 +1,6 @@
 package Treex::Block::Read::BaseReader;
 {
-  $Treex::Block::Read::BaseReader::VERSION = '0.08330_1';
+  $Treex::Block::Read::BaseReader::VERSION = '0.08590_1';
 }
 use Moose;
 use Treex::Core::Common;
@@ -54,6 +54,7 @@ sub is_next_document_for_this_job {
 }
 
 sub next_filename {
+    
     my ($self) = @_;
 
     # In parallel processing and one_doc_per_file setting,
@@ -63,6 +64,9 @@ sub next_filename {
         $self->_set_file_number( $self->file_number + 1 );
         $self->_set_doc_number( $self->doc_number + 1 );
     }
+    # return undef, but do not move further if we are at the end of document list (we might need the current file name) 
+    return if ( $self->file_number >= $self->from->number_of_files );
+    
     $self->_set_file_number( $self->file_number + 1 );
     return $self->current_filename();
 }
@@ -70,6 +74,7 @@ sub next_filename {
 use File::Spec;
 
 sub new_document {
+    
     my ( $self, $load_from ) = @_;
     my $path = $self->current_filename();
     log_fatal "next_filename() must be called before new_document()" if !defined $path;
@@ -136,7 +141,7 @@ Treex::Block::Read::BaseReader - abstract ancestor for document readers
 
 =head1 VERSION
 
-version 0.08330_1
+version 0.08590_1
 
 =head1 DESCRIPTION
 
