@@ -1,6 +1,6 @@
 package Treex::Block::Write::Text;
 {
-  $Treex::Block::Write::Text::VERSION = '0.08330_1';
+  $Treex::Block::Write::Text::VERSION = '0.08590_1';
 }
 use Moose;
 use Treex::Core::Common;
@@ -10,15 +10,18 @@ has '+language' => ( required => 1 );
 
 has '+extension' => ( default => '.txt' );
 
-sub process_document {
+override 'process_document' => sub {
+  
     my ( $self, $doc ) = @_;
 
     $self->_prepare_file_handle($doc);    # open the output file handle
 
     my $doczone = $doc->get_zone( $self->language, $self->selector );
-    print $doczone->text;
+  
+    print { $self->_file_handle } $doczone->text;
+  
     return;
-}
+};
 
 1;
 
@@ -30,7 +33,7 @@ Treex::Block::Write::Text
 
 =head1 VERSION
 
-version 0.08330_1
+version 0.08590_1
 
 =head1 DESCRIPTION
 
@@ -63,10 +66,10 @@ Saves the document.
 
 =head1 AUTHOR
 
-Martin Popel
+Martin Popel <popel@ufal.mff.cuni.cz>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright © 2011 by Institute of Formal and Applied Linguistics, Charles University in Prague
+Copyright © 2011-2012 by Institute of Formal and Applied Linguistics, Charles University in Prague
 
 This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
