@@ -1,6 +1,6 @@
 package Treex::Core::Block;
 {
-  $Treex::Core::Block::VERSION = '0.08330_1';
+  $Treex::Core::Block::VERSION = '0.08590_1';
 }
 use Moose;
 use Treex::Core::Common;
@@ -55,14 +55,17 @@ sub zone_label {
 
 sub BUILD {
     my $self = shift;
-    $self->require_files_from_share( $self->get_required_share_files() );
+
     return;
 }
 
 sub require_files_from_share {
     my ( $self, @rel_paths ) = @_;
     my $my_name = 'the block ' . $self->get_block_name();
-    return map { Treex::Core::Resource::require_file_from_share( $_, $my_name ) } @rel_paths;
+    return map {
+        log_info $self->get_block_name() . " requires file " . $_;
+        Treex::Core::Resource::require_file_from_share( $_, $my_name )
+    } @rel_paths;
 }
 
 sub get_required_share_files {
@@ -156,6 +159,14 @@ sub process_zone {
     return;
 }
 
+sub process_start {
+    my ($self) = @_;
+
+    $self->require_files_from_share( $self->get_required_share_files() );
+
+    return;
+}
+
 sub process_end {
     my ($self) = @_;
 
@@ -182,7 +193,7 @@ Treex::Core::Block - the basic data-processing unit in the Treex framework
 
 =head1 VERSION
 
-version 0.08330_1
+version 0.08590_1
 
 =head1 SYNOPSIS
 
@@ -263,6 +274,11 @@ However, C<process_pnode> is executed also on the root node
 
 =back
 
+=head2 $block->process_start();
+
+This method is called before all documents are processed.
+This method is responsible for loading required models.
+
 =head2 $block->process_end();
 
 This method is called after all documents are processed.
