@@ -1,8 +1,10 @@
 package Treex::Core::Bundle;
 {
-  $Treex::Core::Bundle::VERSION = '0.08330_1';
+  $Treex::Core::Bundle::VERSION = '0.08590_1';
 }
 
+use namespace::autoclean;
+
 use Moose;
 use Treex::Core::Common;
 use MooseX::NonMoose;
@@ -267,6 +269,11 @@ sub get_attr {
     return $self->{$attr_name};
 }
 
+# ------- other -------------
+
+sub following {
+    Treex::Core::Node::following(@_);
+}
 
 __PACKAGE__->meta->make_immutable;
 
@@ -285,7 +292,7 @@ Treex::Core::Bundle - a set of equivalent sentences in the Treex framework
 
 =head1 VERSION
 
-version 0.08330_1
+version 0.08590_1
 
 =head1 DESCRIPTION
 
