@@ -1,6 +1,6 @@
 package Treex::Core::Run;
 {
-  $Treex::Core::Run::VERSION = '0.08330_1';
+  $Treex::Core::Run::VERSION = '0.08590_1';
 }
 use 5.008;
 use Moose;
@@ -15,6 +15,7 @@ use File::Temp qw(tempdir);
 use File::Which;
 use List::MoreUtils qw(first_index);
 use IO::Interactive;
+use Time::HiRes;
 use Exporter;
 use base 'Exporter';
 our @EXPORT_OK = q(treex);
@@ -200,6 +201,14 @@ has 'dump_scenario' => (
     documentation => 'Just dump (print to STDOUT) the given scenario and exit.',
 );
 
+has 'dump_required_files' => (
+    traits        => ['Getopt'],
+    is            => 'rw',
+    isa           => 'Bool',
+    default       => 0,
+    documentation => 'Just dump (print to STDOUT) files required by the given scenario and exit.',
+);
+
 has 'survive' => (
     traits        => ['Getopt'],
     is            => 'rw',
@@ -273,7 +282,7 @@ sub BUILD {
 
 sub _execute {
     my ($self) = @_;
-    if ( $self->dump_scenario ) {
+    if ( $self->dump_scenario || $self->dump_required_files ) {
 
         # TODO: execute_locally does the same work as the following line in a more safe ways
         # (If someone wants to run treex -d My::Block my_scen.scen)
@@ -287,8 +296,25 @@ sub _execute {
         #my @block_items = Treex::Core::Scenario::parse_scenario_string($scen_str);
         #my @block_items = $dummy_scenario->parse_scenario_string($scen_str);
 
-        print "# Full Scenario generated by 'treex --dump_scenario' on " . localtime() . "\n";
-        print $self->scenario->construct_scenario_string( multiline => 1 ), "\n";
+        if ( $self->dump_scenario ) {
+            print "# Full Scenario generated by 'treex --dump_scenario' on " . localtime() . "\n";
+            print $self->scenario->construct_scenario_string( multiline => 1 ), "\n";
+        }
+
+        if ( $self->dump_required_files ) {
+            print "# Required files generated by 'treex --dump_required_files' on " . localtime() . "\n";
+            print join "\n", $self->scenario->get_required_files(), "\n";
+        }
+        exit;
+    }
+
+    if ( $self->dump_required_files ) {
+
+        my $scen_str = join ' ', @{ $self->extra_argv };
+        $self->set_scenario( Treex::Core::Scenario->new( scenario_string => $scen_str, runner => $self ) );
+
+        print "# Required files generated by 'treex --dump_required_files' on " . localtime() . "\n";
+        print join "\n", $self->scenario->get_required_files(), "\n";
         exit;
     }
 
@@ -348,6 +374,7 @@ my %READER_FOR = (
     'streex'   => 'Treex',
     'mrg'      => 'PennMrg',
     'mrg.gz'   => 'PennMrg',
+    'tag'      => 'CdtTag',
 
     # TODO:
     # conll  => 'Conll',
@@ -370,7 +397,12 @@ sub _get_reader_name_for {
             if ( !defined $first ) {
                 $first = $current;
             }
-            log_fatal 'All files (' . join( ',', @names ) . ') must have the same extension' if ( $current ne $first );
+            if ( $current ne $first ) {
+                log_fatal 'All files (' . join( ',', @names ) . ') must have the same extension' . "\n" .
+                    "    current = $current\n" .
+                    "    first   = $first\n" .
+                    "    curname = $name";
+            }
             push @extensions, $current;
         }
         else {
@@ -635,11 +667,14 @@ sub _print_output_files {
     my ( $self, $doc_number ) = @_;
 
     foreach my $stream (qw(stderr stdout)) {
-        my $mask = $self->workdir . "/output/job*-doc" . sprintf( "%07d", $doc_number ) . ".$stream";
-        my ($filename) = glob $mask;
-        if ( !defined $filename ) {
-            my $message = "Document $doc_number finished without producing $mask . " .
-                " It might be useful to inspect " . $self->workdir . "/output/job*-loading.stderr";
+        my $job_number = $self->_get_job_number_from_doc_number($doc_number);
+
+        my $filename = $self->workdir . "/output/job" . sprintf("%03d", $job_number ) . "-doc" . sprintf( "%07d", $doc_number ) . ".$stream";
+        log_info "Processing output file: " . $filename;
+
+        if ( ! -f $filename ) {
+            my $message = "Document $doc_number finished without producing $filename. " .
+                " It might be useful to inspect " . $self->workdir . "/output/job" . sprintf("%03d", $job_number ) . "-loading.stderr";
             if ( $self->survive ) {
                 log_warn("$message (fatal error ignored due to survival mode, be careful)");
                 return;
@@ -649,6 +684,22 @@ sub _print_output_files {
             }
         }
 
+        # we have to wait until file is really written
+        my $wait_it = 0;
+        if ( -s $filename == 0 ) {
+            `touch $filename`;
+        }
+        #while ( -s $filename == 0 && $wait_it < 1 ) {
+        #
+            # print STDERR "Size: " . ( -s $filename) . "\n";
+            #open my $FILE, '<:encoding(utf8)', $filename;
+            #my $line = <$FILE>;
+            #close $FILE;
+            #log_info "File is still empty. " . $filename;
+        #    Time::HiRes::usleep(300000);
+        #    $wait_it++;
+        #}
+
         open my $FILE, '<:encoding(utf8)', $filename or log_fatal $!;
         if ( $stream eq "stdout" ) {
             while (<$FILE>) {
@@ -689,13 +740,10 @@ sub _print_output_files {
 
 sub _doc_started {
     my ( $self, $doc_number ) = @_;
-    my $mask = $self->workdir . sprintf( '/output/job*-doc%07d.stderr', $doc_number );
+    my $job_number = $self->_get_job_number_from_doc_number($doc_number);
+    my $filename = $self->workdir . sprintf( '/output/job%03d-doc%07d.stderr', $job_number, $doc_number );
 
-    # Note that return glob $mask; does not work
-    # glob in scalar context iterates over files,
-    # so if the file was already "iterated", it return undef.
-    my @filenames = glob($mask);
-    return scalar @filenames;
+    return -f $filename;
 }
 
 sub _wait_for_jobs {
@@ -705,24 +753,50 @@ sub _wait_for_jobs {
     my $total_doc_number    = 0;
     my $all_jobs_finished   = 0;
     my $done                = 0;
+    my $jobs_finished       = 1;
+
+    my $glob_time = 0;
+    my $sleep_time = 0;
+    my $print_time = 0;
+
+    my $wait_time = 1;
 
     while ( !$done ) {
         $total_doc_number    ||= $self->_read_total_doc_number();
-        $all_jobs_finished   ||= ( scalar( () = glob $self->workdir . "/output/job???.finished" ) == $self->jobs );
+
+#        print STDERR "Fin: $all_jobs_finished; JobsFin: $jobs_finished; CurrD: $current_doc_number; TotD: $total_doc_number\n";
+        my $glob_start = Time::HiRes::time();
+
+        if ( ! $all_jobs_finished ) {
+            while ( -f $self->workdir . sprintf("/output/job%03d.finished", $jobs_finished) ) {
+                $jobs_finished++;
+            }
+            $all_jobs_finished = ( $jobs_finished > $self->jobs );
+        }
         $current_doc_started ||= $self->_doc_started($current_doc_number);
 
+        $glob_time += (Time::HiRes::time() - $glob_start);
+
         # If a job starts processing another doc,
         # it means it has finished the current doc.
         my $current_doc_finished = $all_jobs_finished;
         $current_doc_finished ||= $self->_doc_started( $current_doc_number + $self->jobs );
 
         if ($current_doc_finished) {
+            my $print_start = Time::HiRes::time();
             $self->_print_output_files($current_doc_number);
+            $print_time += (Time::HiRes::time() - $print_start);
             $current_doc_number++;
             $current_doc_started = 0;
+            $wait_time = 1 + $total_doc_number / $current_doc_number;
+            if ( $wait_time > 120 ) {
+                $wait_time = 120;
+            }
         }
         else {
-            sleep 1;
+            my $sleep_start = Time::HiRes::time();
+            sleep $wait_time;
+            $sleep_time += (Time::HiRes::time() - $sleep_start);
         }
 
         $self->_check_job_errors;
@@ -734,6 +808,10 @@ sub _wait_for_jobs {
         # the output of the last doc was not forwarded yet.
         $done = $all_jobs_finished && $current_doc_number > $total_doc_number;
     }
+
+    log_info "Glob Time: $glob_time";
+    log_info "Print Time: $print_time";
+    log_info "Sleep Time: $sleep_time";
     return;
 }
 
@@ -916,6 +994,13 @@ sub _redirect_output {
     return;
 }
 
+sub _get_job_number_from_doc_number
+{
+    my ($self, $doc_number) = @_;
+    return ( $doc_number - 1 ) % ($self->jobs) + 1;
+}
+
+
 # not a method !
 sub treex {
 
@@ -981,7 +1066,7 @@ Treex::Core::Run + treex - applying Treex blocks and/or scenarios on data
 
 =head1 VERSION
 
-version 0.08330_1
+version 0.08590_1
 
 =head1 SYNOPSIS
 
@@ -1022,7 +1107,7 @@ create new runner and runs scenario given in parameters
 
 =head1 USAGE
 
- usage: treex [-?dEegjLpqSsv] [long options...] scenario [-- treex_files]
+ usage: treex [-?dEegjLmpqSsv] [long options...] scenario [-- treex_files]
  scenario is a sequence of blocks or *.scen files
  options:
  	-? --usage --help            Prints this usage information.
@@ -1055,13 +1140,18 @@ create new runner and runs scenario given in parameters
  	--outdir                     Not to be used manually. Dictory for
  	                             collecting standard and error outputs in
  	                             parallelized processing.
- 	--qsub                       Additional parameters passed to qsub.
- 	                             Requires -p.
  	--local                      Run jobs locally (might help with
  	                             multi-core machines). Requires -p.
- 	--priority                   Priority for qsub (an integer in the
- 	                             range -1023 to 1024, default=0).
- 	                             Requires -p.
+ 	--priority                   Priority for qsub, an integer in the
+ 	                             range -1023 to 0 (or 1024 for admins),
+ 	                             default=-100. Requires -p.
+ 	--memory -m --mem            How much memory should be allocated for
+ 	                             cluster jobs, default=2G. Requires -p.
+ 	                             Translates to "qsub -hard -l
+ 	                             mem_free=$mem -l act_mem_free=$mem -l
+ 	                             h_vmem=$mem".
+ 	--qsub                       Additional parameters passed to qsub.
+ 	                             Requires -p. See --priority and --mem.
  	--watch                      re-run when the given file is changed
  	                             TODO better doc
  	--workdir                    working directory for temporary files in
@@ -1071,6 +1161,8 @@ create new runner and runs scenario given in parameters
  	                             are created)
  	-d --dump_scenario           Just dump (print to STDOUT) the given
  	                             scenario and exit.
+ 	--dump_required_files        Just dump (print to STDOUT) files
+ 	                             required by the given scenario and exit.
  	--survive                    Continue collecting jobs' outputs even
  	                             if some of them crashed (risky, use with
  	                             care!).
