@@ -1,6 +1,6 @@
 package Treex::Core::Scenario;
 {
-  $Treex::Core::Scenario::VERSION = '0.08330_1';
+  $Treex::Core::Scenario::VERSION = '0.08590_1';
 }
 use Moose;
 use Treex::Core::Common;
@@ -76,7 +76,7 @@ has _global_params => (
 
 has parser => (
     is            => 'ro',
-    isa           => 'Parse::RecDescent',
+    isa           => 'Parse::RecDescent::_Runtime',
     init_arg      => undef,
     builder       => '_build_parser',
     documentation => q{Parses treex scenarios}
@@ -210,6 +210,22 @@ sub construct_scenario_string {
     return join $delim, @block_strings;
 }
 
+
+# reverse of parse_scenario_string, used in Treex::Core::Run for treex --dump
+sub get_required_files {
+    my $self        = shift;
+    my @block_items = @{ $self->block_items };
+    my @required_files;
+    foreach my $block_item (@block_items) {
+        my $block = $self->_load_block($block_item);
+        push @required_files,
+            map {
+                $block_item->{block_name} . "\t" . $_;
+            } $block->get_required_share_files();
+    }
+    return @required_files;
+}
+
 sub _add_quotes {    # adding quotes only if param. value contains a space
     my ($block_parameter) = @_;
     my ( $name, $value ) = split /=/, $block_parameter, 2;
@@ -262,6 +278,11 @@ sub run {
     my $number_of_documents = $reader->number_of_documents_per_this_job() || '?';
     my $document_number     = 0;
 
+    log_info "Applying process_start";
+    foreach my $block ( @{ $self->loaded_blocks } ) {
+        $block->process_start();
+    }
+
     while ( my $document = $reader->next_document_for_this_job() ) {
         $document_number++;
         my $doc_name = $document->full_filename;
@@ -320,7 +341,7 @@ Treex::Core::Scenario - a larger Treex processing unit, composed of blocks
 
 =head1 VERSION
 
-version 0.08330_1
+version 0.08590_1
 
 =head1 SYNOPSIS
 
